#ifndef _HavranRayCaster_H__
#define _HavranRayCaster_H__

#include "RayCaster.h"
#include "Containers.h"
#include <string>
#include "ktbconf.h"
#include "raypack.h"

namespace GtpVisibilityPreprocessor {


class Intersectable;
class VssRay;
class KdTree;
class Ray;
class SimpleRayContainer;
class AxisAlignedBox3;
class Vector3;
struct VssRayContainer;
class Preprocessor;
struct SimpleRay;
class CKTB;
  

/** This class provides an interface for ray casting.
*/
class HavranRayCaster: public RayCaster
{
public:
  /** Default constructor initialising e.g., KD tree
   */
  HavranRayCaster(const Preprocessor &preprocessor);
  virtual ~HavranRayCaster();

  void Build(ObjectContainer &objlist);
  
  int Type() const { return HAVRAN_RAYCASTER; }
  
  virtual int CastRay(
		      const SimpleRay &simpleRay,
		      VssRayContainer &vssRays,
		      const AxisAlignedBox3 &box,
		      const bool castDoubleRay,
		      const bool pruneInvalidRays = true
		      );
  
  virtual void CastRays16(SimpleRayContainer &rays, 
			  VssRayContainer &vssRays,
			  const AxisAlignedBox3 &sbox,
			  const bool castDoubleRay,
			  const bool pruneInvalidRays = true
			  );

  virtual void CastRays16(SimpleRayContainer &rays,
			  int offset,
			  VssRayContainer &vssRays,
			  const AxisAlignedBox3 &sbox,
			  const bool castDoubleRay,
			  const bool pruneInvalidRays = true
			  );  
  void
  CastSimpleForwardRays(SimpleRayContainer &rays,
			const AxisAlignedBox3 &sbox
			);

  virtual void CastRays(
			SimpleRayContainer &rays,
			VssRayContainer &vssRays,
			const AxisAlignedBox3 &sbox,
			const bool castDoubleRay,
			const bool pruneInvalidRays = true);

  // Using packet of 4 rays supposing that these are coherent
  // We give a box to which each ray is clipped to before the
  // ray shooting is computed !
  virtual void CastRaysPacket4(const Vector3 &minBox,
							   const Vector3 &maxBox,
							   const Vector3 origin4[],
							   const Vector3 direction4[],
							   int     result4[],
							   float   dist4[]);

#ifdef _USE_HAVRAN_SSE  
  // Just for testing concept
  virtual void CastRaysPacket2x2(RayPacket2x2 &raysPack,
				 bool castDoubleRay,
				 const bool pruneInvalidRays = true);
#endif  

  bool ExportBinTree(const string &filename);
  bool ImportBinTree(const string &filename, ObjectContainer &objects);
  
protected:
  CKTB *mKtbtree;
  static RayPacket2x2 raypack;
};


} // namespace

#endif
